@@ -235,6 +235,7 @@ class SeriesInt64(SeriesAbstractNumeric):
     **Database support and types**
 
     * Postgres: utilizes the 'bigint' database type.
+    * Athena: utilizes the 'bigint' database type.
     * BigQuery: utilizes the 'INT64' database type.
     """
     dtype = 'int64'
@@ -245,6 +246,7 @@ class SeriesInt64(SeriesAbstractNumeric):
         DBDialect.BIGQUERY: 'INT64'
     }
     supported_value_types = (int, numpy.int64, numpy.int32)
+    supported_source_dtypes = ('float64', 'bool', 'string')
 
     @classmethod
     def supported_literal_to_expression(cls, dialect: Dialect, literal: Expression) -> Expression:
@@ -280,7 +282,7 @@ class SeriesInt64(SeriesAbstractNumeric):
     def dtype_to_expression(cls, dialect: Dialect, source_dtype: str, expression: Expression) -> Expression:
         if source_dtype == 'int64':
             return expression
-        if source_dtype not in ['float64', 'bool', 'string']:
+        if source_dtype not in cls.supported_source_dtypes:
             raise ValueError(f'cannot convert {source_dtype} to int64')
         return Expression.construct(f'cast({{}} as {cls.get_db_dtype(dialect)})', expression)
 
@@ -351,6 +353,7 @@ class SeriesFloat64(SeriesAbstractNumeric):
     **Database support and types**
 
     * Postgres: utilizes the 'double precision' database type.
+    * Athena: utilizes the 'double' database type.
     * BigQuery: utilizes the 'FLOAT64' database type.
     """
     dtype = 'float64'
@@ -361,6 +364,7 @@ class SeriesFloat64(SeriesAbstractNumeric):
         DBDialect.BIGQUERY: 'FLOAT64'
     }
     supported_value_types = (float, numpy.float64)
+    supported_source_dtypes = ('int64', 'string')
 
     # Notes for supported_value_to_literal() and supported_literal_to_expression():
     #
@@ -402,7 +406,7 @@ class SeriesFloat64(SeriesAbstractNumeric):
     def dtype_to_expression(cls, dialect: Dialect, source_dtype: str, expression: Expression) -> Expression:
         if source_dtype == 'float64':
             return expression
-        if source_dtype not in ['int64', 'string']:
+        if source_dtype not in cls.supported_source_dtypes:
             raise ValueError(f'cannot convert {source_dtype} to float64')
         return Expression.construct(f'cast({{}} as {cls.get_db_dtype(dialect)})', expression)
 
